@@ -26,11 +26,18 @@ Created by bmuller <bmuller@butterfat.net>
 */
 
 #include "mod_auth_openid.h"
+#ifdef AP_DECLARE_MODULE
+AP_DECLARE_MODULE(authopenid_module);
+#endif
 
 #define APDEBUG(r, msg, ...) ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r, msg, __VA_ARGS__);
 #define APWARN(r, msg, ...) ap_log_rerror(APLOG_MARK, APLOG_WARNING, 0, r, msg, __VA_ARGS__);
 #define APERR(r, msg, ...) ap_log_rerror(APLOG_MARK, APLOG_ERR, 0, r, msg, __VA_ARGS__);
 
+#ifndef APR_ARRAY_IDX
+#define APR_ARRAY_IDX(ary,i,type) (((type *)(ary)->elts)[i])
+#endif
+
 extern "C" module AP_MODULE_DECLARE_DATA authopenid_module;
 
 typedef struct {
@@ -319,7 +326,7 @@ static bool is_distrusted_provider(modauthopenid_config *s_cfg, std::string url,
 };
 
 static bool has_valid_session(request_rec *r, modauthopenid_config *s_cfg) {
-  // test for valid session - if so, return DECLINED
+  // test for valid session
   std::string session_id = "";
   modauthopenid::get_session_id(r, std::string(s_cfg->cookie_name), session_id);
   if(session_id != "" && s_cfg->use_cookie) {
@@ -557,7 +564,7 @@ static int validate_authentication_session(request_rec *r, modauthopenid_config
     // if we're not setting cookie - don't redirect, just show page
     APDEBUG(r, "Setting REMOTE_USER to %s", remote_user.c_str());
     r->user = apr_pstrdup(r->pool, remote_user.c_str());
-    return DECLINED;
+    return OK;
   } catch(opkele::exception &e) {
     APERR(r, "Error in authentication: %s", e.what());
     consumer.close();
@@ -608,6 +615,54 @@ static int mod_authopenid_method_handler(request_rec *r) {
   }
 }
 
+#if AP_MODULE_MAGIC_AT_LEAST(20080403,1)
+static authz_status user_check_authorization(request_rec *r,
+                                             const char *require_args,
+                                             const void *parsed_require_args)
+{
+    const char *t, *w;
+
+    if (!r->user) {
+        return AUTHZ_DENIED_NO_USER;
+    }
+
+    t = require_args;
+    while ((w = ap_getword_conf(r->pool, &t)) && w[0]) {
+        if (!strcmp(r->user, w)) {
+            return AUTHZ_GRANTED;
+        }
+    }
+
+    ap_log_rerror(APLOG_MARK, APLOG_ERR, 0, r, APLOGNO(01663)
+                  "access to %s failed, reason: user '%s' does not meet "
+                  "'require'ments for user to be allowed access",
+                  r->uri, r->user);
+
+    return AUTHZ_DENIED;
+}
+
+static authz_status validuser_check_authorization(request_rec *r,
+                                                  const char *require_line,
+                                                  const void *parsed_require_line)
+{
+    if (!r->user) {
+        return AUTHZ_DENIED_NO_USER;
+    }
+
+    return AUTHZ_GRANTED;
+}
+
+static const authz_provider authz_user_provider =
+{
+    &user_check_authorization,
+    NULL,
+};
+static const authz_provider authz_validuser_provider =
+{
+    &validuser_check_authorization,
+    NULL,
+};
+#else
 static int mod_authopenid_check_user_access(request_rec *r) {
   modauthopenid_config *s_cfg;
   s_cfg = (modauthopenid_config *) ap_get_module_config(r->per_dir_config, &authopenid_module);
@@ -649,10 +704,31 @@ static int mod_authopenid_check_user_access(request_rec *r) {
   ap_note_auth_failure(r);
   return HTTP_UNAUTHORIZED;
 }
+#endif
 
 static void mod_authopenid_register_hooks (apr_pool_t *p) {
+#if AP_MODULE_MAGIC_AT_LEAST(20080403,1)
+  ap_hook_check_authn(mod_authopenid_method_handler,
+                                NULL,
+                                NULL,
+                                APR_HOOK_MIDDLE,
+                                AP_AUTH_INTERNAL_PER_CONF);
+  ap_register_auth_provider(p,
+                                AUTHZ_PROVIDER_GROUP,
+                                "valid-user",
+                                AUTHZ_PROVIDER_VERSION,
+                                &authz_validuser_provider,
+                                AP_AUTH_INTERNAL_PER_CONF);
+  ap_register_auth_provider(p,
+                                AUTHZ_PROVIDER_GROUP,
+                                "user",
+                                AUTHZ_PROVIDER_VERSION,
+                                &authz_user_provider,
+                                AP_AUTH_INTERNAL_PER_CONF);
+#else
   ap_hook_check_user_id(mod_authopenid_method_handler, NULL, NULL, APR_HOOK_MIDDLE);
   ap_hook_auth_checker(mod_authopenid_check_user_access, NULL, NULL, APR_HOOK_MIDDLE);
+#endif
 }
 
 //module AP_MODULE_DECLARE_DATA 
